﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectExcavator
{
    public class DrawingExcavator
    {
        /// <summary>
        /// Класс-сущность
        /// </summary>
        public EntityExcavator? EntityExcavator { get; private set; }
        /// <summary>
        /// Ширина окна
        /// </summary>
        private int _pictureWidth;
        /// <summary>
        /// Высота окна
        /// </summary>
        private int _pictureHeight;
        /// <summary>
        /// /// Левая координата прорисовки автомобиля
        /// </summary>
        private int _startPosX;
        /// <summary>
        /// Верхняя кооридната прорисовки автомобиля
        /// </summary>
        private int _startPosY;
        /// <summary>
        /// Ширина прорисовки автомобиля
        /// </summary>
        private int _exWidth;
        /// <summary>
        /// Высота прорисовки автомобиля
        /// </summary>
        private int _exHeight;
        /// <summary>
        /// Инициализация свойств
        /// </summary>
        /// <param name="speed">Скорость</param>
        /// <param name="weight">Вес</param>
        /// <param name="bodyColor">Цвет кузова</param>
        /// <param name="additionalColor">Дополнительный цвет</param>
        /// <param name="width">Ширина картинки</param>
        /// <param name="height">Высота картинки</param>
        /// <param name="kovsh">Ковш</param>
        /// <param name="katki">Катки гусеничные</param>
        /// <returns>true - объект создан, false - проверка не пройдена, нельзя создать объект в этих размерах</returns>
        public bool Init(int speed, double weight, Color bodyColor, Color additionalColor, bool kovsh, bool katki, int width, int height)
        {
            if (width < _pictureWidth || height < _pictureHeight)
            {
                return false;
            }

            if (!kovsh)
            {
                _exWidth = 105;
                _exHeight = 75;
            }
            else
            {
                _exWidth = 140;
                _exHeight = 87;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            EntityExcavator = new EntityExcavator();
            EntityExcavator.Init(speed, weight, bodyColor, additionalColor, kovsh, katki);
            return true;
        }
        /// <summary>
        /// Установка позиции
        /// </summary>
        /// <param name="x">Координата X</param>
        /// <param name="y">Координата Y</param>
        public void SetPosition(int x, int y)
        {
            // TODO: Изменение x, y
            if (x < 0)
            {
                x = 0;
            }
            else if (x > _pictureWidth - _exWidth)
            {
                x = _pictureWidth - _exWidth;
            }

            if (y < 0)
            {
                y = 0;
            }
            else if (y > _pictureHeight - _exHeight)
            {
                y = _pictureHeight - _exHeight;
            }
            _startPosX = x;
            _startPosY = y;
        }
        /// <summary>
        /// Изменение направления перемещения
        /// </summary>
        /// <param name="direction">Направление</param>
        public void MoveTransport(DirectionType direction)
        {
            if (EntityExcavator == null)
            {
                return;
            }
            switch (direction)
            {
                //влево
                case DirectionType.Left:
                    if (_startPosX - EntityExcavator.Step > 0)
                    {
                        _startPosX -= (int)EntityExcavator.Step;
                    }
                    else
                    {
                        _startPosX = 0;
                    }
                    break;
                //вверх
                case DirectionType.Up:
                    if (_startPosY - EntityExcavator.Step > 0)
                    {
                        _startPosY -= (int)EntityExcavator.Step;
                    }
                    else
                    {
                        _startPosY = 0;
                    }
                    break;
                //вправо
                case DirectionType.Right:
                    if (_startPosX + _exWidth + EntityExcavator.Step <= _pictureWidth)
                    {
                        _startPosX += (int)EntityExcavator.Step;
                    }
                    else
                    {
                        _startPosX = _pictureWidth - _exWidth;
                    }
                    break;
                //вниз
                case DirectionType.Down:
                    if (_startPosY + _exHeight + EntityExcavator.Step <= _pictureHeight)
                    {
                        _startPosY += (int)EntityExcavator.Step;
                    }
                    else
                    {
                        _startPosY = _pictureHeight - _exHeight;
                    }
                    break;
            }
        }
        /// <summary>
        /// Прорисовка объекта
        /// </summary>
        /// <param name="g"></param>
        public void DrawTransport(Graphics g)
        {
            if (EntityExcavator == null)
            {
                return;
            }
            Pen pen = new(Color.Black);
            Brush additionalBrush = new
            SolidBrush(EntityExcavator.AdditionalColor);
            //цвета
            Brush brBlue = new SolidBrush(Color.LightBlue);
            Brush brYellow = new SolidBrush(Color.Yellow);
            Brush brGray = new SolidBrush(Color.Gray);
            Brush brBlack = new SolidBrush(Color.Black);
            //отрисовка экскаватора без ковша
            if (!EntityExcavator.Kovsh)
            {
                g.DrawRectangle(pen, _startPosX + 15, _startPosY + 25, 75, 25);
                g.DrawRectangle(pen, _startPosX + 60, _startPosY, 30, 25);
                g.DrawRectangle(pen, _startPosX + 30, _startPosY + 5, 10, 20);
                g.DrawRectangle(pen, _startPosX + 10, _startPosY + 55, 86, 20);
                g.DrawPie(pen, _startPosX, _startPosY + 55, 20, 20, 90, 180);
                g.DrawPie(pen, _startPosX + 85, _startPosY + 55, 20, 20, 270, 180);
                g.DrawEllipse(pen, _startPosX + 5, _startPosY + 58, 15, 15);
                g.DrawEllipse(pen, _startPosX + 85, _startPosY + 58, 15, 15);
                g.DrawEllipse(pen, _startPosX + 25, _startPosY + 65, 8, 8);
                g.DrawEllipse(pen, _startPosX + 45, _startPosY + 65, 8, 8);
                g.DrawEllipse(pen, _startPosX + 65, _startPosY + 65, 8, 8);
                g.DrawEllipse(pen, _startPosX + 37, _startPosY + 58, 6, 6);
                g.DrawEllipse(pen, _startPosX + 57, _startPosY + 58, 6, 6);
                //кабина водителя
                g.FillRectangle(brBlue, _startPosX + 61, _startPosY + 1, 29, 24);
                // кузов
                g.FillRectangle(brYellow, _startPosX + 16, _startPosY + 26, 74, 24);
                // труба
                g.FillRectangle(brYellow, _startPosX + 31, _startPosY + 6, 9, 19);
                //гусеница
                g.FillPie(brGray, _startPosX, _startPosY + 55, 20, 20, 90, 180);
                g.FillPie(brGray, _startPosX + 85, _startPosY + 55, 20, 20, 270, 180);
                g.FillRectangle(brGray, _startPosX + 10, _startPosY + 55, 86, 20);
                g.FillEllipse(brBlack, _startPosX + 5, _startPosY + 58, 15, 15);
                g.FillEllipse(brBlack, _startPosX + 85, _startPosY + 58, 15, 15);
                g.FillEllipse(brBlack, _startPosX + 25, _startPosY + 65, 8, 8);
                g.FillEllipse(brBlack, _startPosX + 45, _startPosY + 65, 8, 8);
                g.FillEllipse(brBlack, _startPosX + 65, _startPosY + 65, 8, 8);
                g.FillEllipse(brBlack, _startPosX + 37, _startPosY + 58, 6, 6);
                g.FillEllipse(brBlack, _startPosX + 57, _startPosY + 58, 6, 6);
            }
            //отрисовка экскаватора с ковшом
            if (EntityExcavator.Kovsh)
            {
                //экскаватор
                g.DrawRectangle(pen, _startPosX + 50, _startPosY + 35, 75, 25);
                g.DrawRectangle(pen, _startPosX + 95, _startPosY + 10, 30, 25);
                g.DrawRectangle(pen, _startPosX + 60, _startPosY + 15, 10, 20);
                g.DrawRectangle(pen, _startPosX + 44, _startPosY + 65, 86, 20);
                g.DrawPie(pen, _startPosX + 34, _startPosY + 65, 20, 20, 90, 180);
                g.DrawPie(pen, _startPosX + 120, _startPosY + 65, 20, 20, 270, 180);
                g.DrawEllipse(pen, _startPosX + 40, _startPosY + 68, 15, 15);
                g.DrawEllipse(pen, _startPosX + 120, _startPosY + 68, 15, 15);
                g.DrawEllipse(pen, _startPosX + 60, _startPosY + 76, 8, 8);
                g.DrawEllipse(pen, _startPosX + 80, _startPosY + 76, 8, 8);
                g.DrawEllipse(pen, _startPosX + 100, _startPosY + 76, 8, 8);
                g.DrawEllipse(pen, _startPosX + 72, _startPosY + 68, 6, 6);
                g.DrawEllipse(pen, _startPosX + 92, _startPosY + 68, 6, 6);
                //кабина водителя
                g.FillRectangle(brBlue, _startPosX + 96, _startPosY + 11, 29, 24);
                // кузов
                g.FillRectangle(brYellow, _startPosX + 51, _startPosY + 36, 74, 24);
                // труба
                g.FillRectangle(brYellow, _startPosX + 61, _startPosY + 16, 9, 19);
                //гусеница
                g.FillPie(brGray, _startPosX + 34, _startPosY + 65, 20, 20, 90, 180);
                g.FillPie(brGray, _startPosX + 120, _startPosY + 65, 20, 20, 270, 180);
                g.FillRectangle(brGray, _startPosX + 44, _startPosY + 65, 86, 20);
                g.FillEllipse(brBlack, _startPosX + 40, _startPosY + 68, 15, 15);
                g.FillEllipse(brBlack, _startPosX + 120, _startPosY + 68, 15, 15);
                g.FillEllipse(brBlack, _startPosX + 60, _startPosY + 76, 8, 8);
                g.FillEllipse(brBlack, _startPosX + 80, _startPosY + 76, 8, 8);
                g.FillEllipse(brBlack, _startPosX + 100, _startPosY + 76, 8, 8);
                g.FillEllipse(brBlack, _startPosX + 72, _startPosY + 68, 6, 6);
                g.FillEllipse(brBlack, _startPosX + 92, _startPosY + 68, 6, 6);
                //ковш
                g.DrawLine(pen, _startPosX + 50, _startPosY + 35, _startPosX + 10, _startPosY + 10);
                g.DrawLine(pen, _startPosX + 58, _startPosY + 35, _startPosX + 12, _startPosY + 5);
                g.DrawEllipse(pen, _startPosX + 7, _startPosY + 4, 7, 7);
                g.DrawLine(pen, _startPosX + 10, _startPosY + 10, _startPosX + 10, _startPosY + 45);
                g.DrawLine(pen, _startPosX + 14, _startPosY + 5, _startPosX + 14, _startPosY + 45);
                g.DrawPie(pen, _startPosX, _startPosY + 44, 28, 30, 90, 180);
                g.DrawLine(pen, _startPosX + 14, _startPosY + 5, _startPosX + 14, _startPosY);
                g.DrawLine(pen, _startPosX + 14, _startPosY, _startPosX + 7, _startPosY + 10);
                g.DrawLine(pen, _startPosX + 14, _startPosY, _startPosX + 50, _startPosY + 12);
                g.DrawLine(pen, _startPosX + 14, _startPosY, _startPosX + 50, _startPosY + 16);
                g.DrawLine(pen, _startPosX + 50, _startPosY + 12, _startPosX + 50, _startPosY + 29);

                g.FillEllipse(additionalBrush, _startPosX + 7, _startPosY + 4, 7, 7);
                g.FillPie(brBlack, _startPosX, _startPosY + 44, 28, 30, 90, 180);
                Point point1 = new Point(_startPosX + 50, _startPosY + 35);
                Point point2 = new Point(_startPosX + 10, _startPosY + 10);
                Point point3 = new Point(_startPosX + 12, _startPosY + 5);
                Point point4 = new Point(_startPosX + 58, _startPosY + 35);
                Point[] truba_1 = { point1, point2, point3, point4, point1 };
                g.FillPolygon(additionalBrush, truba_1);

                Point point5 = new Point(_startPosX + 10, _startPosY + 10);
                Point point6 = new Point(_startPosX + 10, _startPosY + 45);
                Point point7 = new Point(_startPosX + 14, _startPosY + 45);
                Point point8 = new Point(_startPosX + 14, _startPosY + 5);
                Point[] truba_2 = { point5, point6, point7, point8, point5 };
                g.FillPolygon(additionalBrush, truba_2);

                Point point9 = new Point(_startPosX + 14, _startPosY + 5);
                Point point10 = new Point(_startPosX + 14, _startPosY);
                Point point11 = new Point(_startPosX + 7, _startPosY + 10);
                Point[] triangle = { point9, point10, point11, point9 };
                g.FillPolygon(additionalBrush, triangle);

                Point point12 = new Point(_startPosX + 14, _startPosY);
                Point point13 = new Point(_startPosX + 50, _startPosY + 12);
                Point point14 = new Point(_startPosX + 50, _startPosY + 16);
                Point point15 = new Point(_startPosX + 14, _startPosY);
                Point[] krepl = { point12, point13, point14, point15, point12 };
                g.FillPolygon(additionalBrush, krepl);
            }
            //катки гусеничные
            if (!EntityExcavator.Katki && EntityExcavator.Kovsh)
            {
                g.FillEllipse(additionalBrush, _startPosX + 40, _startPosY + 68, 15, 15);
                g.FillEllipse(additionalBrush, _startPosX + 120, _startPosY + 68, 15, 15);
                g.FillEllipse(additionalBrush, _startPosX + 60, _startPosY + 76, 8, 8);
                g.FillEllipse(additionalBrush, _startPosX + 80, _startPosY + 76, 8, 8);
                g.FillEllipse(additionalBrush, _startPosX + 100, _startPosY + 76, 8, 8);
                g.FillEllipse(additionalBrush, _startPosX + 72, _startPosY + 68, 6, 6);
                g.FillEllipse(additionalBrush, _startPosX + 92, _startPosY + 68, 6, 6);
            }
        }
    }
}
